// /tmp/build-stella/stella-3.1.0/native/cpp/stella/primal.hh

/*--------------------------- BEGIN LICENSE BLOCK ---------------------------+
|                                                                            |
| Version: MPL 1.1/GPL 2.0/LGPL 2.1                                          |
|                                                                            |
| The contents of this file are subject to the Mozilla Public License        |
| Version 1.1 (the "License"); you may not use this file except in           |
| compliance with the License. You may obtain a copy of the License at       |
| http://www.mozilla.org/MPL/                                                |
|                                                                            |
| Software distributed under the License is distributed on an "AS IS" basis, |
| WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License   |
| for the specific language governing rights and limitations under the       |
| License.                                                                   |
|                                                                            |
| The Original Code is the STELLA Programming Language.                      |
|                                                                            |
| The Initial Developer of the Original Code is                              |
| UNIVERSITY OF SOUTHERN CALIFORNIA, INFORMATION SCIENCES INSTITUTE          |
| 4676 Admiralty Way, Marina Del Rey, California 90292, U.S.A.               |
|                                                                            |
| Portions created by the Initial Developer are Copyright (C) 1996-2002      |
| the Initial Developer. All Rights Reserved.                                |
|                                                                            |
| Contributor(s):                                                            |
|                                                                            |
| Alternatively, the contents of this file may be used under the terms of    |
| either the GNU General Public License Version 2 or later (the "GPL"), or   |
| the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),   |
| in which case the provisions of the GPL or the LGPL are applicable instead |
| of those above. If you wish to allow use of your version of this file only |
| under the terms of either the GPL or the LGPL, and not to allow others to  |
| use your version of this file under the terms of the MPL, indicate your    |
| decision by deleting the provisions above and replace them with the notice |
| and other provisions required by the GPL or the LGPL. If you do not delete |
| the provisions above, a recipient may use your version of this file under  |
| the terms of any one of the MPL, the GPL or the LGPL.                      |
|                                                                            |
+---------------------------- END LICENSE BLOCK ----------------------------*/



namespace stella {

// Global declarations:
extern boolean TRUE;
extern boolean FALSE;
extern double PI;
extern int NULL_INTEGER;
extern short int NULL_SHORT_INTEGER;
extern long int NULL_LONG_INTEGER;
extern unsigned short int NULL_UNSIGNED_SHORT_INTEGER;
extern unsigned long int NULL_UNSIGNED_LONG_INTEGER;
extern double NULL_FLOAT;
extern float NULL_SINGLE_FLOAT;
extern double NULL_DOUBLE_FLOAT;
extern char NULL_CHARACTER;
extern char NULL_BYTE;
extern char NULL_OCTET;
extern DECLARE_STELLA_SPECIAL(oTRANSIENTOBJECTSpo, boolean );
extern int* oHASH_BYTE_RANDOM_TABLEo;
extern Vector* oHASH_TABLE_SIZE_PRIMESo;

// Function signatures:
boolean eqlP(Object* x, Object* y);
boolean equalP(Object* x, Object* y);
boolean zeroP(int x);
boolean plusP(int x);
boolean evenP(int x);
boolean oddP(int x);
int mod(int x, int modulo);
int random_(int n);
void seedRandomNumberGenerator();
double sqrt_(double n);
double cos_(double n);
double sin_(double n);
double tan_(double n);
double log_(double n);
int min_(int x, int y);
int max_(int x, int y);
int integerAbs(int x);
double floatAbs(double x);
int characterCode(char ch);
char codeCharacter(int code);
char characterDowncase(char ch);
char characterUpcase(char ch);
char characterCapitalize(char ch);
boolean stringEqlP(char* x, char* y);
boolean stringEqualP(char* x, char* y);
int stringCompare(char* x, char* y, boolean caseSensitiveP);
boolean stringL(char* x, char* y);
boolean stringLE(char* x, char* y);
boolean stringGE(char* x, char* y);
boolean stringG(char* x, char* y);
boolean stringLessP(char* x, char* y);
boolean stringLessEqualP(char* x, char* y);
boolean stringGreaterEqualP(char* x, char* y);
boolean stringGreaterP(char* x, char* y);
char* makeMutableString(int size, char initchar);
char* makeRawMutableString(int size);
char* stringConcatenate(char* string1, char* string2, int otherstrings, ...);
char* replaceSubstrings(char* string, char* neW, char* old);
int insertString(char* source, int start, int end, char* target, int targetIndex, Keyword* caseConversion);
char* characterToString(char c);
char* formatWithPadding(char* input, int length, char padchar, Keyword* align, boolean truncateP);
char stringFirst(char* self);
char mutableStringFirst(char* self);
char mutableStringFirstSetter(char* self, char ch);
char stringSecond(char* self);
char mutableStringSecond(char* self);
char mutableStringSecondSetter(char* self, char ch);
char stringThird(char* self);
char mutableStringThird(char* self);
char mutableStringThirdSetter(char* self, char ch);
char stringFourth(char* self);
char mutableStringFourth(char* self);
char mutableStringFourthSetter(char* self, char ch);
char stringFifth(char* self);
char mutableStringFifth(char* self);
char mutableStringFifthSetter(char* self, char ch);
char stringNth(char* self, int position);
char mutableStringNth(char* self, int position);
char mutableStringNthSetter(char* self, char ch, int position);
int stringLength(char* self);
int mutableStringLength(char* self);
Object* unstringify(char* string);
int objectHashCode(Object* self);
int stringHashCode(char* self);
int integerHashCode(int self);
int floatHashCode(double self);
int characterHashCode(char self);
int hashString(char* string, int seedcode);
int pickHashTableSizePrime(int minsize);
void resizeVector(Vector* self, int size);
Object* nativeVectorNth(Object** self, int position);
Object* nativeVectorNthSetter(Object** self, Object* value, int position);
void unmake(Object* self);
void copyFile(char* fromfile, char* tofile);
int lognot(int arg);
int logand(int arg1, int arg2);
int logor(int arg1, int arg2);
int logxor(int arg1, int arg2);
int shiftLeft(int arg, int count);
int shiftRight(int arg, int count);
Object* stellify(Object* self);
boolean stellaObjectP(Object* self);
boolean runningAsLispP();
void sleep_(double seconds);
char* exceptionMessage(exception* e);
void printExceptionContext(exception* e, OutputStream* stream);
char* makeProcessLock();
void helpStartupPrimal1();
void helpStartupPrimal2();
void helpStartupPrimal3();
void helpStartupPrimal4();
void startupPrimal();

// Auxiliary global declarations:
extern Symbol* SYM_PRIMAL_STELLA_FLOOR;
extern Keyword* KWD_PRIMAL_CPP;
extern Keyword* KWD_PRIMAL_FUNCTION;
extern Symbol* SYM_PRIMAL_STELLA_RANDOM;
extern Symbol* SYM_PRIMAL_STELLA_SQRT;
extern Symbol* SYM_PRIMAL_STELLA_COS;
extern Symbol* SYM_PRIMAL_STELLA_SIN;
extern Symbol* SYM_PRIMAL_STELLA_TAN;
extern Symbol* SYM_PRIMAL_STELLA_LOG;
extern Symbol* SYM_PRIMAL_STELLA_MIN;
extern Symbol* SYM_PRIMAL_STELLA_MAX;
extern Keyword* KWD_PRIMAL_UPCASE;
extern Keyword* KWD_PRIMAL_DOWNCASE;
extern Keyword* KWD_PRIMAL_CAPITALIZE;
extern Keyword* KWD_PRIMAL_PRESERVE;
extern Keyword* KWD_PRIMAL_LEFT;
extern Keyword* KWD_PRIMAL_RIGHT;
extern Keyword* KWD_PRIMAL_CENTER;
extern Symbol* SYM_PRIMAL_STELLA_HASH_CODE;
extern Keyword* KWD_PRIMAL_JAVA;
extern Symbol* SYM_PRIMAL_STELLA_SLEEP;
extern Symbol* SYM_PRIMAL_STELLA_STARTUP_PRIMAL;
extern Symbol* SYM_PRIMAL_STELLA_METHOD_STARTUP_CLASSNAME;


} // end of namespace stella
